<?php

namespace Drupal\core_views_facets\EventSubscriber;

use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\views\Ajax\ViewAjaxResponse;
use Drupal\Core\Plugin\PluginBase;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Response subscriber to handle AJAX responses.
 */
class AjaxResponseSubscriber implements EventSubscriberInterface {

  /**
   * Renders the ajax commands right before preparing the result.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The response event, which contains the possible AjaxResponse object.
   */
  public function onResponse(FilterResponseEvent $event) {
    $response = $event->getResponse();

    // Only alter views ajax responses.
    if (!($response instanceof ViewAjaxResponse)) {
      return;
    }

    $view = $response->getView();

    // Views removes processed arguments and parameters from the request,
    // so every facet but the first one would be wrong.
    $view_parameters = $view->getRequest()->query->all();

    $views_arguments = &drupal_static('core_views_contextual_filter_ajax_arguments');
    $views_arguments = $view->args;

    drupal_static('core_views_exposed_filter_ajax_rendered_status', TRUE);
    drupal_static('core_views_contextual_filter_ajax_rendered_status', TRUE);

    /** @var \Drupal\facets\FacetManager\DefaultFacetManager $facet_manager */
    $facet_manager = \Drupal::service('facets.manager');
    $exposed_facets = $facet_manager->getFacetsByFacetSourceId('core_views_exposed_filter' . PluginBase::DERIVATIVE_SEPARATOR . $view->id() . '__' . $view->current_display);

    if (empty($exposed_facets)) {
      $rendered_status = &drupal_static('core_views_exposed_filter_ajax_rendered_status');
      $rendered_status = FALSE;
    }

    $contextual_facets = $facet_manager->getFacetsByFacetSourceId('core_views_contextual_filter' . PluginBase::DERIVATIVE_SEPARATOR . $view->id() . '__' . $view->current_display);

    if (empty($contextual_facets)) {
      $rendered_status = &drupal_static('core_views_contextual_filter_ajax_rendered_status');
      $rendered_status = FALSE;
    }

    /** @var \Drupal\facets\FacetInterface[] $facets */
    $facets = array_merge($exposed_facets, $contextual_facets);
    foreach ($facets as $facet) {
      \Drupal::request()->query->add($view_parameters);
      $build = $facet_manager->build($facet);
      if (empty($build)) {
        $build = ['#markup' => '<span data-drupal-facet-id="' . $facet->id() . '" />'];
      }
      $response->addCommand(new ReplaceCommand('[data-drupal-facet-id="' . $facet->id() . '"]', $build));
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [KernelEvents::RESPONSE => [['onResponse']]];
  }

}
